/**
 * Copyright 2007 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */
package org.jasig.web.servlet.mvc;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.Validate;
import org.jasig.web.portlet.mvc.AbstractAjaxController;
import org.jasig.web.util.DefaultModelPasser;
import org.jasig.web.util.ModelPasser;
import org.springframework.web.bind.ServletRequestUtils;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.AbstractController;

/**
 * Retrieves the model from the porlet using {@link ModelPasser#getModelFromPortlet(HttpServletRequest, HttpServletResponse, String)}
 * then renders the specified view with the model. If no model is found a ServletException is thrown.
 * 
 * By default requireSession is set to true and the viewName is set to {@link #DEFAULT_VIEW_NAME}
 * 
 * @author Eric Dalquist
 * @version $Revision$
 */
public class AjaxResponseController extends AbstractController {
    public static final String SESSION_KEY = "key";
    
    public static final String DEFAULT_VIEW_NAME = "jsonView";
    
    private ModelPasser modelPasser = new DefaultModelPasser();
    private String viewName = DEFAULT_VIEW_NAME;
    
    public AjaxResponseController() {
        this.setRequireSession(true);
    }
    
    
    public ModelPasser getModelPasser() {
        return this.modelPasser;
    }
    /**
     * Used to get the model data from the data portlet. Defaults to {@link DefaultModelPasser}
     */
    public void setModelPasser(ModelPasser modelPasser) {
        Validate.notNull(modelPasser);
        this.modelPasser = modelPasser;
    }
    
    public String getViewName() {
        return this.viewName;
    }
    /**
     * The view used to render the model. Defaults to {@link #DEFAULT_VIEW_NAME}
     */
    public void setViewName(String viewName) {
        Validate.notNull(viewName);
        this.viewName = viewName;
    }


    /* (non-Javadoc)
     * @see org.springframework.web.servlet.mvc.AbstractController#handleRequestInternal(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
     */
    @Override
    protected ModelAndView handleRequestInternal(HttpServletRequest request, HttpServletResponse response) throws Exception {
        final String key = ServletRequestUtils.getRequiredStringParameter(request, SESSION_KEY);
        
        final Map<Object, Object> model = this.modelPasser.getModelFromPortlet(request, response, key);
        
        //If not model send a 404 status to signal no data was found
        if (model == null) {
            response.sendError(HttpServletResponse.SC_NOT_FOUND, "No model exists in the session for key '" + key + "'");
        }
        //If an error response send a 500 status code to signal an error
        else if (model.containsKey(AbstractAjaxController.DEFAULT_AJAX_HANDLER_ERROR_KEY)) {
            response.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
        }

        return new ModelAndView(this.viewName, model);
    }
}
